﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace OpacityMaskTester
{
    /// <summary>
    /// JogWheel control 
    /// 11/2008 Rob Davis - created for Positech2000 but completely reusable
    /// no restrictions on reuse
    /// </summary>
    public partial class JogWheel : UserControl
    {
        private bool MouseDown; // track is mouse is pressed
        private Double LastAngle; // record last angle
        private bool FirstHemi; // which hemisphere are we in? need to apply restrictions
        public bool Restrict; // if set, wheel is not turnable beyond 360 degrees
        public double Angle
        {
            get { return LastAngle; }
        }

        public Double Range; // value between 0.0 - 1.0 where 0.0 is at 0 degrees and 1.0 is 360 degrees, never actually makes to 0.0 or 1.0

        // allow parent to subscribe to update event
        public delegate void Update_EventHandler(object sender, MouseEventArgs e);
        public event Update_EventHandler Update_Event;

        public JogWheel() {
            Init();
        }

        public JogWheel(Brush TintBrush) {
            Init();
            Tint.Fill = TintBrush;
        }

        private void Init() {
            InitializeComponent();
            FingerDepression.MouseLeftButtonDown += new MouseButtonEventHandler(FingerDepression_MouseLeftButtonDown);
            FingerDepression.MouseLeftButtonUp += new MouseButtonEventHandler(FingerDepression_MouseLeftButtonUp);
            FingerDepression.MouseMove += new MouseEventHandler(FingerDepression_MouseMove);
            FingerDepression.LostMouseCapture += new MouseEventHandler(FingerDepression_LostMouseCapture);
            ResetAngle(0);
        }

        /// <summary>
        /// lose control of the mouse reset the mouse down flag
        /// </summary>
        void FingerDepression_LostMouseCapture(object sender, MouseEventArgs e) {
            MouseDown = false;
        }

        /// <summary>
        /// mouse movement tracking, ignored if mouse button is not down
        /// </summary>
        /// <param name="sender">FingerDepression</param>
        /// <param name="e">MouseEventArgs</param>
        void FingerDepression_MouseMove(object sender, MouseEventArgs e) {
            if (!MouseDown) return;
            Point MouseLoc = e.GetPosition(this);
            Point MouseOff = e.GetPosition(FingerDepression);
            Double Radians = Math.Atan2(MouseLoc.Y - 70.0, MouseLoc.X - 70.0);
            Double NewAngle = Radians * (180 / Math.PI);
            NewAngle += 90; // 0 degrees would actually be at the 3o'Clock position, this compensates but means values are  -90 - 270
            SetAngle(NewAngle);
            // fire event to parent, it can then get Range
            Update_Event(this, null);
        }

        /// <summary>
        /// Reset angle and ensure hemisphere data is correct
        /// </summary>
        /// <param name="NewAngle">New Angle</param>
        public void ResetAngle(Double NewAngle) {
            if ((NewAngle >= 180 && NewAngle <= 270) ||(NewAngle<0)) {
                FirstHemi = false;
            }
            if (NewAngle >= 0 && NewAngle < 180) {
                FirstHemi = true;
            }
            SetAngle(NewAngle, false);
        }

        /// <summary>
        /// pass through
        /// </summary>
        /// <param name="NewAngle">New Angle</param>
        private void SetAngle(Double NewAngle) {
            SetAngle(NewAngle, Restrict);
        }

        /// <summary>
        /// Update the visible control
        /// </summary>
        /// <param name="NewAngle">New Angle</param>
        /// <param name="Restrict">Restrict rotation Flag</param>
        private void SetAngle(Double NewAngle, bool Restrict) {
            if ((LastAngle <= 180 && LastAngle >= 90) && (NewAngle >= 180 && NewAngle <= 270)) {
                FirstHemi = false;
            }
            if ((LastAngle <= 270 && LastAngle >= 180) && (NewAngle >= 90 && NewAngle <= 180)) {
                FirstHemi = true;
            }
            if (Restrict) {
                if ((FirstHemi) && (NewAngle <= 0 || NewAngle >= 180)) {
                    return;
                }
                if ((!FirstHemi) && (NewAngle <= 180 && NewAngle >= 0)) {
                    return;
                }
            }
            ((RotateTransform)Rotator.RenderTransform).Angle = NewAngle;
            ((RotateTransform)FingerDepression.RenderTransform).Angle = -NewAngle; // ensures gradient points the correct way
            LastAngle = NewAngle;
            Range=CalculateRange(LastAngle);
        }

        /// <summary>
        /// calculate a value from 0.0 - 1.0 corosponding to 0 degrees round to 360 degrees.
        /// result never matches 0.0 or 1.0 because of double math
        /// </summary>
        /// <param name="NewAngle">New Angle</param>
        /// <returns>0.0 - 1.0</returns>
        private double CalculateRange(double NewAngle) {
            if (NewAngle < 0) NewAngle = 360 + NewAngle;
            return (NewAngle / 360);
        }

        /// <summary>
        /// track mouse state
        /// </summary>
        void FingerDepression_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            MouseDown = false;
        }

        /// <summary>
        /// track mouse state
        /// </summary>
        void FingerDepression_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            MouseDown = true;
            ((UIElement)FingerDepression).CaptureMouse();
        }

        /// <summary>
        /// Allow subscriber to retrive event at anytime.
        /// </summary>
        public void TriggerUpdateEvent() {
            Update_Event(this, null);
        }
    }
}
